@@ -18,14 +18,13 @@
 /* Also need KEY_sigvar */
 #include "keywords.h"
 
-#include "make_argcheck_aux.c.inc"
-
-#include "LOGOP_ANY.c.inc"
-
 #include "parse_subsignature_ex.h"
 
 #include "lexer-additions.c.inc"
 
+#include "LOGOP_ANY.c.inc"
+#include "croak_from_caller.c.inc"
+#include "make_argcheck_aux.c.inc"
 #include "newSV_with_free.c.inc"
 
 #define newSVpvx(ptr)  S_newSVpvx(aTHX_ ptr)
@@ -125,6 +124,11 @@ S_find_runcv_name(pTHX)
   return sv;
 }
 
+enum {
+  OPp_NAMEDARGDEFELEM_IF_UNDEF = 1,
+  OPp_NAMEDARGDEFELEM_IF_FALSE = 2,
+};
+
 static OP *pp_namedargdefelem(pTHX)
 {
   dSP;
@@ -137,7 +141,22 @@ static OP *pp_namedargdefelem(pTHX)
   /* TODO: we could precompute the hash and store it in the ANY vector */
   SV *value = hv_delete_ent(slurpy_hv, keysv, 0, 0);
 
-  if(value) {
+  bool ok = false;
+  switch(PL_op->op_private & 3) {
+    case 0:
+      ok = !!value;
+      break;
+
+    case OPp_NAMEDARGDEFELEM_IF_UNDEF:
+      ok = value && SvOK(value);
+      break;
+
+    case OPp_NAMEDARGDEFELEM_IF_FALSE:
+      ok = value && SvTRUE(value);
+      break;
+  }
+
+  if(ok) {
     EXTEND(SP, 1);
     PUSHs(value);
     RETURN;
@@ -146,7 +165,7 @@ static OP *pp_namedargdefelem(pTHX)
   if(cLOGOP->op_other)
     return cLOGOP->op_other;
 
-  croak("Missing argument '%" SVf "' for subroutine %" SVf,
+  croak_from_caller("Missing argument '%" SVf "' for subroutine %" SVf,
     SVfARG(keysv), SVfARG(S_find_runcv_name(aTHX)));
 }
 
@@ -171,7 +190,7 @@ static OP *pp_checknomorenamed(pTHX)
   while((he = hv_iternext(slurpy_hv)))
     sv_catpvf(keynames, ", '%" SVf "'", SVfARG(HeSVKEY_force(he))), nkeys++;
 
-  croak("Unrecognised %s %" SVf " for subroutine %" SVf,
+  croak_from_caller("Unrecognised %s %" SVf " for subroutine %" SVf,
     nkeys > 1 ? "arguments" : "argument",
     SVfARG(keynames), SVfARG(S_find_runcv_name(aTHX)));
 }
@@ -280,7 +299,7 @@ static OP *S_parse_sigelem(pTHX_ struct SignatureParsingContext *ctx, U32 flags)
   /* Consume sigil */
   lex_read_unichar(0);
 
-  char *lexname_end;
+  char *lexname_end = NULL;
 
   if(isIDFIRST_uni(lex_peek_unichar(0))) {
     lex_read_unichar(0);
@@ -349,7 +368,7 @@ static OP *S_parse_sigelem(pTHX_ struct SignatureParsingContext *ctx, U32 flags)
   }
 
   if(c == '$') {
-    SV *argname;
+    SV *argname = NULL;
 
     if(paramctx.is_named) {
       parser->sig_slurpy = '+';
@@ -362,10 +381,10 @@ static OP *S_parse_sigelem(pTHX_ struct SignatureParsingContext *ctx, U32 flags)
       parser->sig_elems++;
     }
 
-    if(lex_peek_unichar(0) == '=') {
-      lex_read_unichar(0);
-      lex_read_space(0);
-
+    bool default_if_undef = false, default_if_false = false;
+    if(lex_consume("=") ||
+        (default_if_undef = lex_consume("//=")) ||
+        (default_if_false = lex_consume("||="))) {
       if(!paramctx.is_named)
         parser->sig_optelems++;
 
@@ -373,11 +392,28 @@ static OP *S_parse_sigelem(pTHX_ struct SignatureParsingContext *ctx, U32 flags)
 
       if(paramctx.is_named) {
         paramctx.defop = (OP *)alloc_LOGOP_ANY(OP_CUSTOM, defexpr, LINKLIST(defexpr));
+        paramctx.defop->op_private =
+          default_if_undef ? OPp_NAMEDARGDEFELEM_IF_UNDEF :
+          default_if_false ? OPp_NAMEDARGDEFELEM_IF_FALSE :
+                            0;
         paramctx.defop->op_ppaddr = &pp_namedargdefelem;
       }
       else {
+        U8 private = 0;
+#ifdef OPpARG_IF_UNDEF
+        if(default_if_undef) private |= OPpARG_IF_UNDEF;
+        if(default_if_false) private |= OPpARG_IF_FALSE;
+#else
+        if(default_if_undef || default_if_false)
+          /* TODO: This would be possible with a custom op but we'd basically
+           * have to copy the behaviour of pp_argdefelem in that case
+           */
+          yyerror("This Perl version cannot handle if_undef/if_false defaulting expressions on positional parameters");
+#endif
+
         paramctx.defop = (OP *)alloc_LOGOP(OP_ARGDEFELEM, defexpr, LINKLIST(defexpr));
         paramctx.defop->op_targ = (PADOFFSET)(parser->sig_elems - 1);
+        paramctx.defop->op_private = private;
       }
 
       paramctx.varop->op_flags |= OPf_STACKED;
@@ -450,8 +486,6 @@ OP *XPS_parse_subsignature_ex(pTHX_ int flags)
   yy_parser *parser = PL_parser;
   struct SignatureParsingContext ctx = {};
 
-  bool permit_named_params = flags & PARSE_SUBSIGNATURE_NAMED_PARAMS;
-
   assert((flags & ~(PARSE_SUBSIGNATURE_NAMED_PARAMS|PARSE_SUBSIGNATURE_PARAM_ATTRIBUTES)) == 0);
 
   ENTER;
